@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -16,11 +16,12 @@
  */
 package org.apache.qpid.jms.message;
 
+import static org.apache.qpid.jms.message.JmsMessagePropertySupport.convertPropertyTo;
+
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Set;
-import java.util.concurrent.Callable;
 
 import javax.jms.DeliveryMode;
 import javax.jms.Destination;
@@ -29,18 +30,19 @@ import javax.jms.MessageFormatException;
 import javax.jms.MessageNotReadableException;
 import javax.jms.MessageNotWriteableException;
 
+import org.apache.qpid.jms.JmsAcknowledgeCallback;
 import org.apache.qpid.jms.JmsConnection;
 import org.apache.qpid.jms.exceptions.JmsExceptionSupport;
 import org.apache.qpid.jms.message.facade.JmsMessageFacade;
-import org.apache.qpid.jms.util.TypeConversionSupport;
 
 public class JmsMessage implements javax.jms.Message {
 
     private static final String ID_PREFIX = "ID:";
-    protected transient Callable<Void> acknowledgeCallback;
+    protected transient JmsAcknowledgeCallback acknowledgeCallback;
     protected transient JmsConnection connection;
 
     protected final JmsMessageFacade facade;
+    protected boolean readOnly;
     protected boolean readOnlyBody;
     protected boolean readOnlyProperties;
     protected boolean validatePropertyNames = true;
@@ -57,7 +59,7 @@ public class JmsMessage implements javax.jms.Message {
 
     protected void copy(JmsMessage other) {
         this.readOnlyBody = other.readOnlyBody;
-        this.readOnlyProperties = other.readOnlyBody;
+        this.readOnlyProperties = other.readOnlyProperties;
         this.acknowledgeCallback = other.acknowledgeCallback;
         this.connection = other.connection;
         this.validatePropertyNames = other.validatePropertyNames;
@@ -65,7 +67,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public int hashCode() {
-        String id = facade.getMessageId();
+        Object id = facade.getMessageId();
 
         if (id != null) {
             return id.hashCode();
@@ -84,8 +86,8 @@ public class JmsMessage implements javax.jms.Message {
         }
 
         JmsMessage msg = (JmsMessage) o;
-        String oMsg = msg.facade.getMessageId();
-        String thisMsg = facade.getMessageId();
+        Object oMsg = msg.facade.getMessageId();
+        Object thisMsg = facade.getMessageId();
 
         return thisMsg != null && oMsg != null && oMsg.equals(thisMsg);
     }
@@ -94,15 +96,35 @@ public class JmsMessage implements javax.jms.Message {
     public void acknowledge() throws JMSException {
         if (acknowledgeCallback != null) {
             try {
-                acknowledgeCallback.call();
+                acknowledgeCallback.acknowledge();
+                acknowledgeCallback = null;
             } catch (Throwable e) {
                 throw JmsExceptionSupport.create(e);
             }
         }
     }
 
+    @Override
+    public boolean isBodyAssignableTo(@SuppressWarnings("rawtypes") Class target) throws JMSException {
+        return true;
+    }
+
+    @Override
+    public final <T> T getBody(Class<T> asType) throws JMSException {
+        if (isBodyAssignableTo(asType)) {
+            return doGetBody(asType);
+        }
+
+        throw new MessageFormatException("Message body cannot be read as type: " + asType);
+    }
+
+    protected <T> T doGetBody(Class<T> asType) throws JMSException {
+        return null;
+    }
+
     @Override
     public void clearBody() throws JMSException {
+        checkReadOnly();
         readOnlyBody = false;
         facade.clearBody();
     }
@@ -115,6 +137,14 @@ public class JmsMessage implements javax.jms.Message {
         this.validatePropertyNames = validatePropertyNames;
     }
 
+    public boolean isReadOnly() {
+        return this.readOnly;
+    }
+
+    public void setReadOnly(boolean readOnly) {
+        this.readOnly = readOnly;
+    }
+
     public boolean isReadOnlyBody() {
         return this.readOnlyBody;
     }
@@ -133,16 +163,17 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public String getJMSMessageID() throws JMSException {
-        String value = facade.getMessageId();
-        if (value != null && !value.startsWith(ID_PREFIX)) {
-            value = ID_PREFIX + value;
+        String id = facade.getMessageId();
+        if (id != null && !id.startsWith(ID_PREFIX)) {
+            id = ID_PREFIX + id;
         }
 
-        return value;
+        return id;
     }
 
     @Override
     public void setJMSMessageID(String value) throws JMSException {
+        checkReadOnly();
         facade.setMessageId(value);
     }
 
@@ -153,6 +184,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSTimestamp(long timestamp) throws JMSException {
+        checkReadOnly();
         facade.setTimestamp(timestamp);
     }
 
@@ -163,6 +195,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSCorrelationID(String correlationId) throws JMSException {
+        checkReadOnly();
         facade.setCorrelationId(correlationId);
     }
 
@@ -173,6 +206,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSCorrelationIDAsBytes(byte[] correlationId) throws JMSException {
+        checkReadOnly();
         facade.setCorrelationIdBytes(correlationId);
     }
 
@@ -183,6 +217,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSReplyTo(Destination destination) throws JMSException {
+        checkReadOnly();
         facade.setReplyTo(JmsMessageTransformation.transformDestination(connection, destination));
     }
 
@@ -193,6 +228,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSDestination(Destination destination) throws JMSException {
+        checkReadOnly();
         facade.setDestination(JmsMessageTransformation.transformDestination(connection, destination));
     }
 
@@ -203,7 +239,17 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSDeliveryMode(int mode) throws JMSException {
-        facade.setPersistent(mode == DeliveryMode.PERSISTENT);
+        checkReadOnly();
+        switch (mode) {
+            case DeliveryMode.PERSISTENT:
+                facade.setPersistent(true);
+                break;
+            case DeliveryMode.NON_PERSISTENT:
+                facade.setPersistent(false);
+                break;
+            default:
+                throw new JMSException(String.format("Invalid DeliveryMode specific: %d", mode));
+        }
     }
 
     @Override
@@ -213,6 +259,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSRedelivered(boolean redelivered) throws JMSException {
+        checkReadOnly();
         facade.setRedelivered(redelivered);
     }
 
@@ -223,6 +270,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSType(String type) throws JMSException {
+        checkReadOnly();
         facade.setType(type);
     }
 
@@ -233,6 +281,7 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSExpiration(long expiration) throws JMSException {
+        checkReadOnly();
         facade.setExpiration(expiration);
     }
 
@@ -243,42 +292,40 @@ public class JmsMessage implements javax.jms.Message {
 
     @Override
     public void setJMSPriority(int priority) throws JMSException {
+        checkReadOnly();
+
+        if (priority < 0 || priority > 9) {
+            throw new JMSException(String.format("Priority value given {%d} is out of range (0..9)", priority));
+        }
+
         facade.setPriority(priority);
     }
 
+    @Override
+    public long getJMSDeliveryTime() throws JMSException {
+        return facade.getDeliveryTime();
+    }
+
+    @Override
+    public void setJMSDeliveryTime(long deliveryTime) throws JMSException {
+        checkReadOnly();
+        facade.setDeliveryTime(deliveryTime, true);
+    }
+
     @Override
     public void clearProperties() throws JMSException {
-        JmsMessagePropertyIntercepter.clearProperties(facade, true);
+        checkReadOnly();
+        JmsMessagePropertyIntercepter.clearProperties(this, true);
     }
 
     @Override
     public boolean propertyExists(String name) throws JMSException {
-        try {
-            checkPropertyNameIsValid(name);
-        } catch (IllegalArgumentException iae) {
-            return false;
-        }
-
-        return JmsMessagePropertyIntercepter.propertyExists(facade, name);
+        return JmsMessagePropertyIntercepter.propertyExists(this, name);
     }
 
     @Override
     public Enumeration<?> getPropertyNames() throws JMSException {
-        Set<String> result = new HashSet<String>();
-
-        Set<String> propertyNames = JmsMessagePropertyIntercepter.getPropertyNames(facade, true);
-        for (String name : propertyNames) {
-            try {
-                checkPropertyNameIsValid(name);
-            } catch (IllegalArgumentException iae) {
-                // Don't add the name
-                continue;
-            }
-
-            result.add(name);
-        }
-
-        return Collections.enumeration(result);
+        return Collections.enumeration(JmsMessagePropertyIntercepter.getPropertyNames(this, true));
     }
 
     /**
@@ -286,147 +333,64 @@ public class JmsMessage implements javax.jms.Message {
      * properties
      *
      * @return Enumeration of all property names on this message
-     * @throws JMSException
+     *
+     * @throws JMSException if an error occurs while reading the properties from the Message.
      */
     public Enumeration<?> getAllPropertyNames() throws JMSException {
         Set<String> result = new HashSet<String>();
-        result.addAll(JmsMessagePropertyIntercepter.getAllPropertyNames(facade));
+        result.addAll(JmsMessagePropertyIntercepter.getAllPropertyNames(this));
         return Collections.enumeration(result);
     }
 
     @Override
     public void setObjectProperty(String name, Object value) throws JMSException {
-        checkReadOnlyProperties();
-        checkPropertyNameIsValid(name);
-        checkValidObject(value);
-        JmsMessagePropertyIntercepter.setProperty(facade, name, value);
-    }
-
-    protected void checkValidObject(Object value) throws MessageFormatException {
-        boolean valid = value instanceof Boolean ||
-                        value instanceof Byte ||
-                        value instanceof Short ||
-                        value instanceof Integer ||
-                        value instanceof Long ||
-                        value instanceof Float ||
-                        value instanceof Double ||
-                        value instanceof Character ||
-                        value instanceof String ||
-                        value == null;
-
-        if (!valid) {
-            throw new MessageFormatException("Only objectified primitive objects and String types are allowed but was: " + value + " type: " + value.getClass());
-        }
+        checkReadOnly();
+        JmsMessagePropertyIntercepter.setProperty(this, name, value);
     }
 
     @Override
     public Object getObjectProperty(String name) throws JMSException {
-        checkPropertyNameIsValid(name);
-        return JmsMessagePropertyIntercepter.getProperty(facade, name);
+        return JmsMessagePropertyIntercepter.getProperty(this, name);
     }
 
     @Override
     public boolean getBooleanProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            return false;
-        }
-        Boolean rc = (Boolean) TypeConversionSupport.convert(value, Boolean.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a boolean");
-        }
-        return rc.booleanValue();
+        return convertPropertyTo(name, getObjectProperty(name), Boolean.class);
     }
 
     @Override
     public byte getByteProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NumberFormatException("property " + name + " was null");
-        }
-        Byte rc = (Byte) TypeConversionSupport.convert(value, Byte.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a byte");
-        }
-        return rc.byteValue();
+        return convertPropertyTo(name, getObjectProperty(name), Byte.class);
     }
 
     @Override
     public short getShortProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NumberFormatException("property " + name + " was null");
-        }
-        Short rc = (Short) TypeConversionSupport.convert(value, Short.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a short");
-        }
-        return rc.shortValue();
+        return convertPropertyTo(name, getObjectProperty(name), Short.class);
     }
 
     @Override
     public int getIntProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NumberFormatException("property " + name + " was null");
-        }
-        Integer rc = (Integer) TypeConversionSupport.convert(value, Integer.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as an integer");
-        }
-        return rc.intValue();
+        return convertPropertyTo(name, getObjectProperty(name), Integer.class);
     }
 
     @Override
     public long getLongProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NumberFormatException("property " + name + " was null");
-        }
-        Long rc = (Long) TypeConversionSupport.convert(value, Long.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a long");
-        }
-        return rc.longValue();
+        return convertPropertyTo(name, getObjectProperty(name), Long.class);
     }
 
     @Override
     public float getFloatProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NullPointerException("property " + name + " was null");
-        }
-        Float rc = (Float) TypeConversionSupport.convert(value, Float.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a float");
-        }
-        return rc.floatValue();
+        return convertPropertyTo(name, getObjectProperty(name), Float.class);
     }
 
     @Override
     public double getDoubleProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            throw new NullPointerException("property " + name + " was null");
-        }
-        Double rc = (Double) TypeConversionSupport.convert(value, Double.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a double");
-        }
-        return rc.doubleValue();
+        return convertPropertyTo(name, getObjectProperty(name), Double.class);
     }
 
     @Override
     public String getStringProperty(String name) throws JMSException {
-        Object value = getObjectProperty(name);
-        if (value == null) {
-            return null;
-        }
-        String rc = (String) TypeConversionSupport.convert(value, String.class);
-        if (rc == null) {
-            throw new MessageFormatException("Property " + name + " was a " + value.getClass().getName() + " and cannot be read as a String");
-        }
-        return rc;
+        return convertPropertyTo(name, getObjectProperty(name), String.class);
     }
 
     @Override
@@ -469,42 +433,48 @@ public class JmsMessage implements javax.jms.Message {
         setObjectProperty(name, value);
     }
 
-    public Callable<Void> getAcknowledgeCallback() {
+    public JmsAcknowledgeCallback getAcknowledgeCallback() {
         return acknowledgeCallback;
     }
 
-    public void setAcknowledgeCallback(Callable<Void> acknowledgeCallback) {
-        this.acknowledgeCallback = acknowledgeCallback;
+    public void setAcknowledgeCallback(JmsAcknowledgeCallback jmsAcknowledgeCallback) {
+        this.acknowledgeCallback = jmsAcknowledgeCallback;
     }
 
     /**
      * Used to trigger processing required to place the message in a state where it is
      * ready to be written to the wire.  This processing can include such tasks as ensuring
-     * that the proper message headers are set or compressing message bodies etc.
+     * that the proper message headers are set or compressing message bodies etc.  During this
+     * call the message is placed in a read-only mode and will not be returned to a writable
+     * state until send completion is triggered.
      *
-     * @param disableMessageId
-     *        a hint that the user requested not to sent the MessageID if possible.
-     * @param disableTimestamp
-     *        a hint that the user requested not to sent the MessageID if possible.
      * @param producerTtl
      *        the time to live value that the producer was configured with at send time.
      *
      * @throws JMSException if an error occurs while preparing the message for send.
      */
-    public void onSend(boolean disableMessageId, boolean disableTimestamp, long producerTtl) throws JMSException {
-        setReadOnlyBody(true);
-        setReadOnlyProperties(true);
-        facade.onSend(disableMessageId, disableTimestamp, producerTtl);
+    public void onSend(long producerTtl) throws JMSException {
+        setReadOnly(true);
+        facade.onSend(producerTtl);
+    }
+
+    /**
+     * Used to trigger processing required to place the message into a writable state once
+     * again following completion of the send operation.
+     */
+    public void onSendComplete() {
+        setReadOnly(false);
     }
 
     /**
      * Used to trigger processing required before dispatch of a message to its intended
-     * consumer.  This method should perform any needed unmarshal or message property
+     * consumer.  This method should perform any needed decoding or message property
      * processing prior to the message arriving at a consumer.
      *
      * @throws JMSException if an error occurs while preparing the message for dispatch.
      */
     public void onDispatch() throws JMSException {
+        setReadOnly(false);
         setReadOnlyBody(true);
         setReadOnlyProperties(true);
         facade.onDispatch();
@@ -532,14 +502,22 @@ public class JmsMessage implements javax.jms.Message {
         return "JmsMessage { " + facade + " }";
     }
 
+    //----- State validation methods -----------------------------------------//
+
+    protected void checkReadOnly() throws MessageNotWriteableException {
+        if (readOnly) {
+            throw new MessageNotWriteableException("Message is currently read-only");
+        }
+    }
+
     protected void checkReadOnlyProperties() throws MessageNotWriteableException {
-        if (readOnlyProperties) {
+        if (readOnly || readOnlyProperties) {
             throw new MessageNotWriteableException("Message properties are read-only");
         }
     }
 
     protected void checkReadOnlyBody() throws MessageNotWriteableException {
-        if (readOnlyBody) {
+        if (readOnly || readOnlyBody) {
             throw new MessageNotWriteableException("Message body is read-only");
         }
     }
@@ -549,55 +527,4 @@ public class JmsMessage implements javax.jms.Message {
             throw new MessageNotReadableException("Message body is write-only");
         }
     }
-
-    private void checkPropertyNameIsValid(String propertyName) throws IllegalArgumentException {
-        if (propertyName == null) {
-            throw new IllegalArgumentException("Property name must not be null");
-        } else if (propertyName.length() == 0) {
-            throw new IllegalArgumentException("Property name must not be the empty string");
-        }
-
-        if (isValidatePropertyNames()) {
-            checkIdentifierLetterAndDigitRequirements(propertyName);
-            checkIdentifierIsntNullTrueFalse(propertyName);
-            checkIdentifierIsntLogicOperator(propertyName);
-        }
-    }
-
-    private void checkIdentifierIsntLogicOperator(String identifier) {
-        // Identifiers cannot be NOT, AND, OR, BETWEEN, LIKE, IN, IS, or ESCAPE.
-        if ("NOT".equals(identifier) || "AND".equals(identifier) || "OR".equals(identifier) ||
-            "BETWEEN".equals(identifier) || "LIKE".equals(identifier) || "IN".equals(identifier) ||
-            "IS".equals(identifier) || "ESCAPE".equals(identifier)) {
-
-            throw new IllegalArgumentException("Identifier not allowed in JMS: '" + identifier + "'");
-        }
-    }
-
-    private void checkIdentifierIsntNullTrueFalse(String identifier) {
-        // Identifiers cannot be the names NULL, TRUE, and FALSE.
-        if ("NULL".equals(identifier) || "TRUE".equals(identifier) || "FALSE".equals(identifier)) {
-            throw new IllegalArgumentException("Identifier not allowed in JMS: '" + identifier + "'");
-        }
-    }
-
-    private void checkIdentifierLetterAndDigitRequirements(String identifier) {
-        // An identifier is an unlimited-length sequence of letters and digits, the first of
-        // which must be a letter.  A letter is any character for which the method
-        // Character.isJavaLetter returns true.  This includes '_' and '$'.  A letter or digit
-        // is any character for which the method Character.isJavaLetterOrDigit returns true.
-        char startChar = identifier.charAt(0);
-        if (!(Character.isJavaIdentifierStart(startChar))) {
-            throw new IllegalArgumentException("Identifier does not begin with a valid JMS identifier start character: '" + identifier + "' ");
-        }
-
-        // JMS part character
-        int length = identifier.length();
-        for (int i = 1; i < length; i++) {
-            char ch = identifier.charAt(i);
-            if (!(Character.isJavaIdentifierPart(ch))) {
-                throw new IllegalArgumentException("Identifier contains invalid JMS identifier character '" + ch + "': '" + identifier + "' ");
-            }
-        }
-    }
 }
