@@ -20,9 +20,14 @@
  */
 package org.apache.qpid.jms.integration;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.fail;
 
+import java.net.URLDecoder;
+import java.net.URLEncoder;
+
 import javax.jms.Connection;
 import javax.jms.ConnectionFactory;
 import javax.jms.JMSException;
@@ -36,12 +41,18 @@ import org.apache.qpid.jms.transports.TransportSslOptions;
 import org.apache.qpid.jms.transports.TransportSupport;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class SaslIntegrationTest extends QpidJmsTestCase {
 
+    private static final Logger LOG = LoggerFactory.getLogger(SaslIntegrationTest.class);
+
     private static final Symbol ANONYMOUS = Symbol.valueOf("ANONYMOUS");
     private static final Symbol PLAIN = Symbol.valueOf("PLAIN");
     private static final Symbol CRAM_MD5 = Symbol.valueOf("CRAM-MD5");
+    private static final Symbol SCRAM_SHA_1 = Symbol.valueOf("SCRAM-SHA-1");
+    private static final Symbol SCRAM_SHA_256 = Symbol.valueOf("SCRAM-SHA-256");
     private static final Symbol EXTERNAL = Symbol.valueOf("EXTERNAL");
 
     private static final String BROKER_JKS_KEYSTORE = "src/test/resources/broker-jks.keystore";
@@ -68,7 +79,9 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
 
         try (TestAmqpPeer testPeer = new TestAmqpPeer(context, true);) {
             // Expect an EXTERNAL connection
-            testPeer.expectSaslExternalConnect();
+            testPeer.expectSaslExternal();
+            testPeer.expectOpen();
+
             // Each connection creates a session for managing temporary destinations etc
             testPeer.expectBegin();
 
@@ -93,7 +106,9 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
             String user = "user";
             String pass = "qwerty123456";
 
-            testPeer.expectSaslPlainConnect(user, pass, null, null);
+            testPeer.expectSaslPlain(user, pass);
+            testPeer.expectOpen();
+
             // Each connection creates a session for managing temporary destinations etc
             testPeer.expectBegin();
 
@@ -110,11 +125,55 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         }
     }
 
+    @Test(timeout = 20000)
+    public void testSaslPlainConnectionWithURIEncodedCredentials() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            // Expect a PLAIN connection with decoded password from URL encoded value.
+            String user = "user";
+            String pass = " CN24tCa+Hn/av";
+
+            // If double decoded this value results in " CN24tCa Hn/av" as the decoded plus
+            // becomes a valid encoding for a space character and would be removed.
+            String encodedPass = "+CN24tCa%2BHn%2Fav";
+
+            String urlEncodedPassword = URLEncoder.encode(pass, "UTF-8");
+            String urlDecodedPassword = URLDecoder.decode(pass, "UTF-8");
+
+            // Inadvertent double decoding of the password should result in a different value
+            // which would fail this test.
+            assertEquals(encodedPass, urlEncodedPassword);
+            assertFalse(urlEncodedPassword.equals(urlDecodedPassword));
+            assertFalse(pass.equals(urlDecodedPassword));
+
+            testPeer.expectSaslPlain(user, pass);
+            testPeer.expectOpen();
+
+            // Each connection creates a session for managing temporary destinations etc
+            testPeer.expectBegin();
+
+            ConnectionFactory factory = new JmsConnectionFactory(
+                "amqp://localhost:" + testPeer.getServerPort() +
+                "?jms.username=" + user + "&jms.password=" + encodedPass);
+
+            Connection connection = factory.createConnection();
+            // Set a clientID to provoke the actual AMQP connection process to occur.
+            connection.setClientID("clientName");
+
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertNull(testPeer.getThrowable());
+
+            testPeer.expectClose();
+            connection.close();
+        }
+    }
+
     @Test(timeout = 20000)
     public void testSaslAnonymousConnection() throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
             // Expect an ANOYMOUS connection
-            testPeer.expectSaslAnonymousConnect();
+            testPeer.expectSaslAnonymous();
+            testPeer.expectOpen();
             // Each connection creates a session for managing temporary destinations etc
             testPeer.expectBegin();
 
@@ -134,6 +193,8 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     /**
      * Add a small delay after the SASL process fails, test peer will throw if
      * any unexpected frames arrive, such as erroneous open+close.
+     *
+     * @throws Exception if an error occurs during the test.
      */
     @Test(timeout = 20000)
     public void testWaitForUnexpectedFramesAfterSaslFailure() throws Exception {
@@ -155,10 +216,20 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         doMechanismSelectedTestImpl("username", "password", CRAM_MD5, new Symbol[] {CRAM_MD5, PLAIN, ANONYMOUS}, false);
     }
 
+    @Test(timeout = 20000)
+    public void testScramSha1SelectedWhenCredentialsPresent() throws Exception {
+        doMechanismSelectedTestImpl("username", "password", SCRAM_SHA_1, new Symbol[] {SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS}, false);
+    }
+
+    @Test(timeout = 20000)
+    public void testScramSha256SelectedWhenCredentialsPresent() throws Exception {
+        doMechanismSelectedTestImpl("username", "password", SCRAM_SHA_256, new Symbol[] {SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS}, false);
+    }
+
     private void doMechanismSelectedTestImpl(String username, String password, Symbol clientSelectedMech, Symbol[] serverMechs, boolean wait) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
 
-            testPeer.expectFailingSaslConnect(serverMechs, clientSelectedMech);
+            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
 
             ConnectionFactory factory = new JmsConnectionFactory("amqp://localhost:" + testPeer.getServerPort() + "?jms.clientID=myclientid");
             try {
@@ -168,6 +239,8 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
                 // Expected, we deliberately failed the SASL process,
                 // we only wanted to verify the correct mechanism
                 // was selected, other tests verify the remainder.
+
+                LOG.info("Caught expected security exception: {}", jmsse.getMessage());
             }
 
             if (wait) {
@@ -180,12 +253,12 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
 
     @Test(timeout = 20000)
     public void testExternalSelectedWhenLocalPrincipalPresent() throws Exception {
-        doMechanismSelectedExternalTestImpl(true, EXTERNAL, new Symbol[] {EXTERNAL, ANONYMOUS});
+        doMechanismSelectedExternalTestImpl(true, EXTERNAL, new Symbol[] {EXTERNAL, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
     }
 
     @Test(timeout = 20000)
     public void testExternalNotSelectedWhenLocalPrincipalMissing() throws Exception {
-        doMechanismSelectedExternalTestImpl(false, ANONYMOUS, new Symbol[] {EXTERNAL, ANONYMOUS});
+        doMechanismSelectedExternalTestImpl(false, ANONYMOUS, new Symbol[] {EXTERNAL, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
     }
 
     private void doMechanismSelectedExternalTestImpl(boolean requireClientCert, Symbol clientSelectedMech, Symbol[] serverMechs) throws Exception {
@@ -209,7 +282,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
                                "transport.keyStorePassword=" + PASSWORD;
             }
 
-            testPeer.expectFailingSaslConnect(serverMechs, clientSelectedMech);
+            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
 
             JmsConnectionFactory factory = new JmsConnectionFactory("amqps://localhost:" + testPeer.getServerPort() + connOptions);
             try {
@@ -271,7 +344,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     private void doMechanismSelectionRestrictedTestImpl(String username, String password, Symbol clientSelectedMech, Symbol[] serverMechs, String mechanismsOptionValue) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
 
-            testPeer.expectFailingSaslConnect(serverMechs, clientSelectedMech);
+            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
 
             String uriOptions = "?jms.clientID=myclientid";
             if(mechanismsOptionValue != null) {
