@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -25,8 +25,8 @@ import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNotSame;
 import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.io.ByteArrayOutputStream;
@@ -56,8 +56,7 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
     public void testNewMessageToSendContainsMessageTypeAnnotation() throws Exception {
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createNewObjectMessageFacade(false);
 
-        Message protonMessage = amqpObjectMessageFacade.getAmqpMessage();
-        MessageAnnotations annotations = protonMessage.getMessageAnnotations();
+        MessageAnnotations annotations = amqpObjectMessageFacade.getMessageAnnotations();
         Map<Symbol, Object> annotationsMap = annotations.getValue();
 
         assertNotNull("MessageAnnotations section was not present", annotations);
@@ -97,14 +96,13 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
 
     private void doNewMessageToSendHasBodySectionRepresentingNull(boolean amqpTyped) throws Exception {
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createNewObjectMessageFacade(amqpTyped);
-        amqpObjectMessageFacade.onSend(false, false, 0);
+        amqpObjectMessageFacade.onSend(0);
 
-        Message protonMessage = amqpObjectMessageFacade.getAmqpMessage();
-        assertNotNull("Message body should be presents", protonMessage.getBody());
+        assertNotNull("Message body should be presents", amqpObjectMessageFacade.getBody());
         if(amqpTyped) {
-            assertSame("Expected existing body section to be replaced", AmqpTypedObjectDelegate.NULL_OBJECT_BODY, protonMessage.getBody());
+            assertSame("Expected existing body section to be replaced", AmqpTypedObjectDelegate.NULL_OBJECT_BODY, amqpObjectMessageFacade.getBody());
         } else {
-            assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, protonMessage.getBody());
+            assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, amqpObjectMessageFacade.getBody());
         }
     }
 
@@ -113,6 +111,8 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
     /**
      * Test that setting an object on a new message results in the expected
      * content in the body section of the underlying message.
+     *
+     * @throws Exception if an error occurs during the test.
      */
     @Test
     public void testSetObjectOnNewMessage() throws Exception {
@@ -121,8 +121,6 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createNewObjectMessageFacade(false);
         amqpObjectMessageFacade.setObject(content);
 
-        Message protonMessage = amqpObjectMessageFacade.getAmqpMessage();
-
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
         oos.writeObject(content);
@@ -131,7 +129,7 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
         byte[] bytes = baos.toByteArray();
 
         // retrieve the bytes from the underlying message, check they match expectation
-        Section section = protonMessage.getBody();
+        Section section = amqpObjectMessageFacade.getBody();
         assertNotNull(section);
         assertEquals(Data.class, section.getClass());
         assertArrayEquals("Underlying message data section did not contain the expected bytes", bytes, ((Data) section).getValue().getArray());
@@ -140,6 +138,8 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
     /**
      * Test that setting an object on a new message results in the expected
      * content in the body section of the underlying message.
+     *
+     * @throws Exception if an error occurs during the test.
      */
     @Test
     public void testSetObjectOnNewAmqpTypedMessage() throws Exception {
@@ -148,19 +148,19 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createNewObjectMessageFacade(true);
         amqpObjectMessageFacade.setObject(content);
 
-        Message protonMessage = amqpObjectMessageFacade.getAmqpMessage();
-
         // retrieve the body from the underlying message, check it matches expectation
-        Section section = protonMessage.getBody();
+        Section section = amqpObjectMessageFacade.getBody();
         assertNotNull(section);
         assertEquals(AmqpValue.class, section.getClass());
         assertEquals("Underlying message body did not contain the expected content", content, ((AmqpValue) section).getValue());
     }
 
     /**
-    * Test that setting a null object on a message results in the underlying body
-    * section being set with the null object body, ensuring getObject returns null.
-    */
+     * Test that setting a null object on a message results in the underlying body
+     * section being set with the null object body, ensuring getObject returns null.
+     *
+     * @throws Exception if an error occurs during the test.
+     */
     @Test
     public void testSetObjectWithNullClearsExistingBodySection() throws Exception {
         Message protonMessage = Message.Factory.create();
@@ -169,16 +169,18 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
 
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createReceivedObjectMessageFacade(createMockAmqpConsumer(), protonMessage);
 
-        assertNotNull("Expected existing body section to be found", protonMessage.getBody());
+        assertNotNull("Expected existing body section to be found", amqpObjectMessageFacade.getBody());
         amqpObjectMessageFacade.setObject(null);
-        assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, protonMessage.getBody());
+        assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, amqpObjectMessageFacade.getBody());
         assertNull("Expected null object", amqpObjectMessageFacade.getObject());
     }
 
     /**
-    * Test that clearing the body on a message results in the underlying body
-    * section being set with the null object body, ensuring getObject returns null.
-    */
+     * Test that clearing the body on a message results in the underlying body
+     * section being set with the null object body, ensuring getObject returns null.
+     *
+     * @throws Exception if an error occurs during the test.
+     */
     @Test
     public void testClearBodyWithExistingSerializedBodySection() throws Exception {
         Message protonMessage = Message.Factory.create();
@@ -187,16 +189,18 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
 
         AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createReceivedObjectMessageFacade(createMockAmqpConsumer(), protonMessage);
 
-        assertNotNull("Expected existing body section to be found", protonMessage.getBody());
+        assertNotNull("Expected existing body section to be found", amqpObjectMessageFacade.getBody());
         amqpObjectMessageFacade.clearBody();
-        assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, protonMessage.getBody());
+        assertSame("Expected existing body section to be replaced", AmqpSerializedObjectDelegate.NULL_OBJECT_BODY, amqpObjectMessageFacade.getBody());
         assertNull("Expected null object", amqpObjectMessageFacade.getObject());
     }
 
     /**
-    * Test that setting an object on a new message and later getting the value, returns an
-    * equal but different object that does not pick up intermediate changes to the set object.
-    */
+     * Test that setting an object on a new message and later getting the value, returns an
+     * equal but different object that does not pick up intermediate changes to the set object.
+     *
+     * @throws Exception if an error occurs during the test.
+     */
     @Test
     public void testSetThenGetObjectOnSerializedMessageReturnsSnapshot() throws Exception {
         HashMap<String, String> origMap = new HashMap<String, String>();
@@ -275,4 +279,69 @@ public class AmqpJmsObjectMessageFacadeTest extends AmqpJmsMessageTypesTestCase
             // expected
         }
     }
+
+    /**
+     * Test that setting an object on a received message and later getting the value, returns an
+     * equal but different object that does not pick up intermediate changes to the set object.
+     *
+     * @throws Exception if an error occurs during the test.
+     */
+    @Test
+    public void testSetThenGetObjectOnSerializedReceivedMessageNoContentTypeReturnsSnapshot() throws Exception {
+        doTestSetThenGetObjectOnSerializedReceivedMessageReturnsSnapshot(false);
+    }
+
+    @Test
+    public void testSetThenGetObjectOnSerializedReceivedMessageReturnsSnapshot() throws Exception {
+        doTestSetThenGetObjectOnSerializedReceivedMessageReturnsSnapshot(true);
+    }
+
+    @SuppressWarnings("unchecked")
+    private void doTestSetThenGetObjectOnSerializedReceivedMessageReturnsSnapshot(boolean contentType) throws Exception {
+
+        HashMap<String, String> origMap = new HashMap<String, String>();
+        origMap.put("key1", "value1");
+
+        Message message = Message.Factory.create();
+        if (contentType) {
+            message.setContentType(AmqpMessageSupport.SERIALIZED_JAVA_OBJECT_CONTENT_TYPE);
+            message.setBody(new Data(new Binary(getSerializedBytes(origMap))));
+        } else {
+            message.setBody(new AmqpValue(origMap));
+        }
+        AmqpJmsObjectMessageFacade amqpObjectMessageFacade = createReceivedObjectMessageFacade(createMockAmqpConsumer(), message);
+
+        // verify we get a different-but-equal object back
+        Serializable serialized = amqpObjectMessageFacade.getObject();
+        assertTrue("Unexpected object type returned", serialized instanceof Map<?, ?>);
+        Map<String, String> returnedObject1 = (Map<String, String>) serialized;
+        assertNotSame("Expected different objects, due to snapshot being taken", origMap, returnedObject1);
+        assertEquals("Expected equal objects, due to snapshot being taken", origMap, returnedObject1);
+
+        // verify we get a different-but-equal object back when compared to the previously retrieved object
+        Serializable serialized2 = amqpObjectMessageFacade.getObject();
+        assertTrue("Unexpected object type returned", serialized2 instanceof Map<?, ?>);
+        Map<String, String> returnedObject2 = (Map<String, String>) serialized2;
+        assertNotSame("Expected different objects, due to snapshot being taken", returnedObject1, returnedObject2);
+        assertEquals("Expected equal objects, due to snapshot being taken", returnedObject1, returnedObject2);
+
+        // mutate the first returned object
+        returnedObject1.put("key2", "value2");
+
+        // verify the mutated map is a different and not equal object
+        assertNotSame("Expected different objects, due to snapshot being taken", returnedObject1, returnedObject2);
+        assertNotEquals("Expected objects to differ, due to snapshot being taken", returnedObject1, returnedObject2);
+    }
+
+    private static byte[] getSerializedBytes(Serializable value) throws IOException {
+        try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
+             ObjectOutputStream oos = new ObjectOutputStream(baos)) {
+
+            oos.writeObject(value);
+            oos.flush();
+            oos.close();
+
+            return baos.toByteArray();
+        }
+    }
 }
