@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -16,40 +16,52 @@
  */
 package org.apache.qpid.jms.transports;
 
-import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNull;
 
+import javax.net.ssl.SSLContext;
+
 import org.apache.qpid.jms.test.QpidJmsTestCase;
 import org.junit.Test;
+import org.mockito.Mockito;
 
 /**
  * Test for class TransportSslOptions
  */
 public class TransportSslOptionsTest extends QpidJmsTestCase {
 
-    public static final String PASSWORD = "password";
-    public static final String CLIENT_KEYSTORE = "src/test/resources/client-jks.keystore";
-    public static final String CLIENT_TRUSTSTORE = "src/test/resources/client-jks.truststore";
-    public static final String KEYSTORE_TYPE = "jks";
-    public static final String KEY_ALIAS = "myTestAlias";
-    public static final String CONTEXT_PROTOCOL = "TLSv1.1";
-    public static final boolean TRUST_ALL = true;
-    public static final boolean VERIFY_HOST = true;
+    private static final String PASSWORD = "password";
+    private static final String CLIENT_KEYSTORE = "src/test/resources/client-jks.keystore";
+    private static final String CLIENT_TRUSTSTORE = "src/test/resources/client-jks.truststore";
+    private static final String KEYSTORE_TYPE = "jks";
+    private static final String KEY_ALIAS = "myTestAlias";
+    private static final String CONTEXT_PROTOCOL = "TLSv1.1";
+    private static final boolean TRUST_ALL = true;
+    private static final boolean VERIFY_HOST = true;
 
-    public static final int TEST_SEND_BUFFER_SIZE = 128 * 1024;
-    public static final int TEST_RECEIVE_BUFFER_SIZE = TEST_SEND_BUFFER_SIZE;
-    public static final int TEST_TRAFFIC_CLASS = 1;
-    public static final boolean TEST_TCP_NO_DELAY = false;
-    public static final boolean TEST_TCP_KEEP_ALIVE = true;
-    public static final int TEST_SO_LINGER = Short.MAX_VALUE;
-    public static final int TEST_SO_TIMEOUT = 10;
-    public static final int TEST_CONNECT_TIMEOUT = 90000;
+    private static final int TEST_SEND_BUFFER_SIZE = 128 * 1024;
+    private static final int TEST_RECEIVE_BUFFER_SIZE = TEST_SEND_BUFFER_SIZE;
+    private static final int TEST_TRAFFIC_CLASS = 1;
+    private static final boolean TEST_TCP_NO_DELAY = false;
+    private static final boolean TEST_TCP_KEEP_ALIVE = true;
+    private static final int TEST_SO_LINGER = Short.MAX_VALUE;
+    private static final int TEST_SO_TIMEOUT = 10;
+    private static final int TEST_CONNECT_TIMEOUT = 90000;
+    private static final int TEST_DEFAULT_SSL_PORT = 5681;
 
-    public static final String[] ENABLED_PROTOCOLS = new String[] {"TLSv1.2"};
-    public static final String[] DISABLED_PROTOCOLS = new String[] {"SSLv3", "TLSv1.2"};
-    public static final String[] ENABLED_CIPHERS = new String[] {"CIPHER_A", "CIPHER_B"};
-    public static final String[] DISABLED_CIPHERS = new String[] {"CIPHER_C"};
+    private static final String[] ENABLED_PROTOCOLS = new String[] {"TLSv1.2"};
+    private static final String[] DISABLED_PROTOCOLS = new String[] {"SSLv3", "TLSv1.2"};
+    private static final String[] ENABLED_CIPHERS = new String[] {"CIPHER_A", "CIPHER_B"};
+    private static final String[] DISABLED_CIPHERS = new String[] {"CIPHER_C"};
+
+    private static final SSLContext SSL_CONTEXT = Mockito.mock(SSLContext.class);
+
+    private static final String JAVAX_NET_SSL_KEY_STORE = "javax.net.ssl.keyStore";
+    private static final String JAVAX_NET_SSL_KEY_STORE_PASSWORD = "javax.net.ssl.keyStorePassword";
+    private static final String JAVAX_NET_SSL_TRUST_STORE = "javax.net.ssl.trustStore";
+    private static final String JAVAX_NET_SSL_TRUST_STORE_PASSWORD = "javax.net.ssl.trustStorePassword";
 
     @Test
     public void testCreate() {
@@ -70,6 +82,7 @@ public class TransportSslOptionsTest extends QpidJmsTestCase {
         assertNull(options.getTrustStoreLocation());
         assertNull(options.getTrustStorePassword());
         assertNull(options.getKeyAlias());
+        assertNull(options.getSslContextOverride());
     }
 
     @Test
@@ -92,6 +105,7 @@ public class TransportSslOptionsTest extends QpidJmsTestCase {
         assertEquals(KEYSTORE_TYPE, options.getStoreType());
         assertEquals(KEY_ALIAS, options.getKeyAlias());
         assertEquals(CONTEXT_PROTOCOL, options.getContextProtocol());
+        assertEquals(SSL_CONTEXT, options.getSslContextOverride());
         assertArrayEquals(ENABLED_PROTOCOLS,options.getEnabledProtocols());
         assertArrayEquals(DISABLED_PROTOCOLS,options.getDisabledProtocols());
         assertArrayEquals(ENABLED_CIPHERS,options.getEnabledCipherSuites());
@@ -110,6 +124,7 @@ public class TransportSslOptionsTest extends QpidJmsTestCase {
         assertEquals(TEST_SO_LINGER, options.getSoLinger());
         assertEquals(TEST_SO_TIMEOUT, options.getSoTimeout());
         assertEquals(TEST_CONNECT_TIMEOUT, options.getConnectTimeout());
+        assertEquals(TEST_DEFAULT_SSL_PORT, options.getDefaultSslPort());
 
         assertEquals(CLIENT_KEYSTORE, options.getKeyStoreLocation());
         assertEquals(PASSWORD, options.getKeyStorePassword());
@@ -118,6 +133,7 @@ public class TransportSslOptionsTest extends QpidJmsTestCase {
         assertEquals(KEYSTORE_TYPE, options.getStoreType());
         assertEquals(KEY_ALIAS, options.getKeyAlias());
         assertEquals(CONTEXT_PROTOCOL, options.getContextProtocol());
+        assertEquals(SSL_CONTEXT, options.getSslContextOverride());
         assertArrayEquals(ENABLED_PROTOCOLS,options.getEnabledProtocols());
         assertArrayEquals(DISABLED_PROTOCOLS,options.getDisabledProtocols());
         assertArrayEquals(ENABLED_CIPHERS,options.getEnabledCipherSuites());
@@ -149,7 +165,53 @@ public class TransportSslOptionsTest extends QpidJmsTestCase {
         options.setSoLinger(TEST_SO_LINGER);
         options.setSoTimeout(TEST_SO_TIMEOUT);
         options.setConnectTimeout(TEST_CONNECT_TIMEOUT);
+        options.setDefaultSslPort(TEST_DEFAULT_SSL_PORT);
+        options.setSslContextOverride(SSL_CONTEXT);
 
         return options;
     }
+
+    @Test
+    public void testSslSystemPropertiesInfluenceDefaults() {
+        String keystore = "keystore";
+        String keystorePass = "keystorePass";
+        String truststore = "truststore";
+        String truststorePass = "truststorePass";
+
+        setSslSystemPropertiesForCurrentTest(keystore, keystorePass, truststore, truststorePass);
+
+        TransportSslOptions options1 = new TransportSslOptions();
+
+        assertEquals(keystore, options1.getKeyStoreLocation());
+        assertEquals(keystorePass, options1.getKeyStorePassword());
+        assertEquals(truststore, options1.getTrustStoreLocation());
+        assertEquals(truststorePass, options1.getTrustStorePassword());
+
+        keystore +="2";
+        keystorePass +="2";
+        truststore +="2";
+        truststorePass +="2";
+
+        setSslSystemPropertiesForCurrentTest(keystore, keystorePass, truststore, truststorePass);
+
+        TransportSslOptions options2 = new TransportSslOptions();
+
+        assertEquals(keystore, options2.getKeyStoreLocation());
+        assertEquals(keystorePass, options2.getKeyStorePassword());
+        assertEquals(truststore, options2.getTrustStoreLocation());
+        assertEquals(truststorePass, options2.getTrustStorePassword());
+
+        assertNotEquals(options1.getKeyStoreLocation(), options2.getKeyStoreLocation());
+        assertNotEquals(options1.getKeyStorePassword(), options2.getKeyStorePassword());
+        assertNotEquals(options1.getTrustStoreLocation(), options2.getTrustStoreLocation());
+        assertNotEquals(options1.getTrustStorePassword(), options2.getTrustStorePassword());
+    }
+
+    private void setSslSystemPropertiesForCurrentTest(String keystore, String keystorePassword, String truststore, String truststorePassword) {
+        setTestSystemProperty(JAVAX_NET_SSL_KEY_STORE, keystore);
+        setTestSystemProperty(JAVAX_NET_SSL_KEY_STORE_PASSWORD, keystorePassword);
+        setTestSystemProperty(JAVAX_NET_SSL_TRUST_STORE, truststore);
+        setTestSystemProperty(JAVAX_NET_SSL_TRUST_STORE_PASSWORD, truststorePassword);
+    }
+
 }
