@@ -36,6 +36,7 @@ import org.apache.qpid.jms.test.Wait;
 import org.apache.qpid.jms.transports.Transport;
 import org.apache.qpid.jms.transports.TransportListener;
 import org.apache.qpid.jms.transports.TransportOptions;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -44,6 +45,8 @@ import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
 import io.netty.channel.epoll.Epoll;
 import io.netty.channel.epoll.EpollEventLoopGroup;
+import io.netty.util.ResourceLeakDetector;
+import io.netty.util.ResourceLeakDetector.Level;
 
 /**
  * Test basic functionality of the Netty based TCP transport.
@@ -59,7 +62,7 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
     protected final List<ByteBuf> data = new ArrayList<ByteBuf>();
     protected final AtomicInteger bytesRead = new AtomicInteger();
 
-    protected final TransportListener testListener = new NettyTransportListener();
+    protected final TransportListener testListener = new NettyTransportListener(false);
 
     @Test(timeout = 60 * 1000)
     public void testCloseOnNeverConnectedTransport() throws Exception {
@@ -256,13 +259,12 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
             }
 
             assertTrue(Wait.waitFor(new Wait.Condition() {
-
                 @Override
                 public boolean isSatisified() throws Exception {
                     LOG.debug("Checking completion: read {} expecting {}", bytesRead.get(), (FRAME_SIZE * CONNECTION_COUNT));
                     return bytesRead.get() == (FRAME_SIZE * CONNECTION_COUNT);
                 }
-            }));
+            }, 10000, 50));
 
             for (Transport transport : transports) {
                 transport.close();
@@ -297,12 +299,11 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
 
         final Transport connectedTransport = transport;
         assertTrue(Wait.waitFor(new Wait.Condition() {
-
             @Override
             public boolean isSatisified() throws Exception {
                 return !connectedTransport.isConnected();
             }
-        }));
+        }, 10000, 50));
 
         assertTrue(data.isEmpty());
 
@@ -367,12 +368,11 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
             transport.send(sendBuffer);
 
             assertTrue(Wait.waitFor(new Wait.Condition() {
-
                 @Override
                 public boolean isSatisified() throws Exception {
                     return !data.isEmpty();
                 }
-            }));
+            }, 10000, 50));
 
             assertEquals(SEND_BYTE_COUNT, data.get(0).readableBytes());
 
@@ -421,12 +421,11 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
             }
 
             assertTrue(Wait.waitFor(new Wait.Condition() {
-
                 @Override
                 public boolean isSatisified() throws Exception {
                     return bytesRead.get() == (byteCount * iterations);
                 }
-            }));
+            }, 10000, 50));
 
             transport.close();
         }
@@ -466,6 +465,45 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
         }
     }
 
+    @Ignore("Used for checking for transport level leaks, my be unstable on CI.")
+    @Test(timeout = 60 * 1000)
+    public void testSendToClosedTransportFailsButDoesNotLeak() throws Exception {
+        Transport transport = null;
+
+        ResourceLeakDetector.setLevel(Level.PARANOID);
+
+        try (NettyEchoServer server = createEchoServer(createServerOptions())) {
+            server.start();
+
+            int port = server.getServerPort();
+            URI serverLocation = new URI("tcp://localhost:" + port);
+
+            for (int i = 0; i < 256; ++i) {
+                transport = createTransport(serverLocation, testListener, createClientOptions());
+                try {
+                    transport.connect(null);
+                    LOG.info("Connected to server:{} as expected.", serverLocation);
+                } catch (Exception e) {
+                    fail("Should have connected to the server at " + serverLocation + " but got exception: " + e);
+                }
+
+                assertTrue(transport.isConnected());
+
+                ByteBuf sendBuffer = transport.allocateSendBuffer(10 * 1024 * 1024);
+
+                transport.close();
+
+                try {
+                    transport.send(sendBuffer);
+                    fail("Should throw on send of closed transport");
+                } catch (IOException ex) {
+                }
+            }
+
+            System.gc();
+        }
+    }
+
     @Test(timeout = 60 * 1000)
     public void testConnectToServerWithEpollEnabled() throws Exception {
         doTestEpollSupport(true);
@@ -568,13 +606,22 @@ public class NettyTcpTransportTest extends QpidJmsTestCase {
         return new NettyEchoServer(options, needClientAuth);
     }
 
-    private class NettyTransportListener implements TransportListener {
+    public class NettyTransportListener implements TransportListener {
+        final boolean retainDataBufs;
+
+        NettyTransportListener(boolean retainDataBufs) {
+            this.retainDataBufs = retainDataBufs;
+        }
 
         @Override
         public void onData(ByteBuf incoming) {
             LOG.debug("Client has new incoming data of size: {}", incoming.readableBytes());
             data.add(incoming);
             bytesRead.addAndGet(incoming.readableBytes());
+
+            if(retainDataBufs) {
+                incoming.retain();
+            }
         }
 
         @Override
