@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -19,11 +19,9 @@ package org.apache.qpid.jms.session;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
 
 import java.util.concurrent.TimeUnit;
 
-import javax.jms.JMSSecurityException;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
 import javax.jms.Queue;
@@ -82,47 +80,6 @@ public class JmsSessionTest extends AmqpTestSupport {
         session.close();
     }
 
-    @Test(timeout=30000)
-    public void testSessionDoubleCloseWithoutException() throws Exception {
-        connection = createAmqpConnection();
-        connection.start();
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        session.close();
-        session.close();
-    }
-
-    @Test(timeout=30000)
-    public void testConsumerCreateThrowsWhenBrokerStops() throws Exception {
-        connection = createAmqpConnection();
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        Queue queue = session.createQueue(getDestinationName());
-        connection.start();
-
-        stopPrimaryBroker();
-        try {
-            session.createConsumer(queue);
-            fail("Should have thrown an IllegalStateException");
-        } catch (Exception ex) {
-            LOG.info("Caught exception on create consumer: {}", ex);
-        }
-    }
-
-    @Test(timeout=30000)
-    public void testProducerCreateThrowsWhenBrokerStops() throws Exception {
-        connection = createAmqpConnection();
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        Queue queue = session.createQueue(getDestinationName());
-        connection.start();
-
-        stopPrimaryBroker();
-        try {
-            session.createProducer(queue);
-            fail("Should have thrown an IllegalStateException");
-        } catch (Exception ex) {
-            LOG.info("Caught exception on create producer: {}", ex);
-        }
-    }
-
     @Test(timeout=30000)
     public void testCreateTemporaryQueue() throws Exception {
         connection = createAmqpConnection();
@@ -135,16 +92,6 @@ public class JmsSessionTest extends AmqpTestSupport {
         assertEquals(1, broker.getTemporaryQueues().length);
     }
 
-    @Test(timeout=30000)
-    public void testCreateTemporaryQueueNotAuthorized() throws Exception {
-        connection = createAmqpConnection("guest", "password");
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        try {
-            session.createTemporaryQueue();
-            fail("Should have thrown a security exception");
-        } catch (JMSSecurityException jmsse) {}
-    }
-
     @Test(timeout=30000)
     public void testDeleteTemporaryQueue() throws Exception {
         connection = createAmqpConnection();
@@ -180,16 +127,6 @@ public class JmsSessionTest extends AmqpTestSupport {
         assertEquals(1, broker.getTemporaryTopics().length);
     }
 
-    @Test(timeout=30000)
-    public void testCreateTemporaryTopicNotAuthorized() throws Exception {
-        connection = createAmqpConnection("guest", "password");
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        try {
-            session.createTemporaryTopic();
-            fail("Should have thrown a security exception");
-        } catch (JMSSecurityException jmsse) {}
-    }
-
     @Test(timeout=30000)
     public void testDeleteTemporaryTopic() throws Exception {
         connection = createAmqpConnection();
