@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -19,6 +19,7 @@ package org.apache.qpid.jms.transactions;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.net.URI;
 import java.util.ArrayList;
@@ -26,6 +27,7 @@ import java.util.List;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 
+import javax.jms.Connection;
 import javax.jms.DeliveryMode;
 import javax.jms.Destination;
 import javax.jms.Message;
@@ -280,7 +282,8 @@ public class JmsTransactedConsumerTest extends AmqpTestSupport {
 
         // Create a new consumer
         consumer = session.createConsumer(queue);
-        message = (TextMessage) consumer.receive(3000);
+        message = (TextMessage) consumer.receive(5000);
+        assertNotNull(message);
         session.commit();
 
         assertEquals(0, proxy.getQueueSize());
@@ -357,6 +360,22 @@ public class JmsTransactedConsumerTest extends AmqpTestSupport {
             @Override
             public void onConnectionEstablished(URI remoteURI) {
             }
+
+            @Override
+            public void onSessionClosed(Session session, Throwable exception) {
+            }
+
+            @Override
+            public void onConsumerClosed(MessageConsumer consumer, Throwable cause) {
+            }
+
+            @Override
+            public void onProducerClosed(MessageProducer producer, Throwable cause) {
+            }
+
+            @Override
+            public void onRemoteDiscovery(List<URI> remotes) {
+            }
         });
 
         MessageProducer pr = session.createProducer(queue);
@@ -489,4 +508,87 @@ public class JmsTransactedConsumerTest extends AmqpTestSupport {
 
         session.close();
     }
+
+    @Test(timeout = 60000)
+    public void testConsumerClosesAfterItsTXCommits() throws Exception {
+        connection = createAmqpConnection();
+        connection.start();
+
+        Session mgmtSession = connection.createSession(true, Session.AUTO_ACKNOWLEDGE);
+        Queue queue = mgmtSession.createQueue(name.getMethodName());
+
+        // Send a message that will be rolled back.
+        Session senderSession = connection.createSession(true, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = senderSession.createProducer(queue);
+        producer.send(senderSession.createMessage());
+        senderSession.commit();
+        senderSession.close();
+
+        // Consumer the message in a transaction and then roll it back
+        Session txSession = connection.createSession(true, Session.AUTO_ACKNOWLEDGE);
+        MessageConsumer consumer = txSession.createConsumer(queue);
+        Message received = consumer.receive(1000);
+        assertNotNull("Consumer didn't receive the message", received);
+        txSession.rollback();
+        consumer.close();
+
+        // Create an auto acknowledge session and consumer normally.
+        Session nonTxSession = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        consumer = nonTxSession.createConsumer(queue);
+        received = consumer.receive(1000);
+        assertNotNull("receiver3 didn't received the message", received);
+        consumer.close();
+
+        connection.close();
+    }
+
+    @Test(timeout = 90000)
+    public void testConsumerMessagesInOrder() throws Exception {
+
+        final int ITERATIONS = 5;
+        final int MESSAGE_COUNT = 4;
+
+        for (int i = 0; i < ITERATIONS; i++) {
+            LOG.debug("##----------- Iteration {} -----------##", i);
+            Connection consumingConnection = createAmqpConnection();
+            Connection producingConnection = createAmqpConnection();
+
+            Session consumerSession = consumingConnection.createSession(true, Session.SESSION_TRANSACTED);
+            Queue queue = consumerSession.createQueue("my_queue");
+            MessageConsumer consumer = consumerSession.createConsumer(queue);
+
+            Session producerSession = producingConnection.createSession(true, Session.SESSION_TRANSACTED);
+            MessageProducer producer = producerSession.createProducer(queue);
+
+            ArrayList<TextMessage> received = new ArrayList<>();
+
+            try {
+                for (int j = 0; j < MESSAGE_COUNT; j++) {
+                    producer.send(producerSession.createTextMessage("msg" + (j + 1)));
+                }
+
+                producerSession.commit();
+
+                consumingConnection.start();
+
+                for (int k = 0; k < MESSAGE_COUNT; k++) {
+                    TextMessage tm = (TextMessage) consumer.receive(5000);
+                    assertNotNull(tm);
+                    received.add(tm);
+                }
+
+                for (int l = 0; l < MESSAGE_COUNT; l++) {
+                    TextMessage tm = received.get(l);
+                    assertNotNull(tm);
+                    if (!("msg" + (l + 1)).equals(tm.getText())) {
+                        fail("Out of order, expected " + ("msg" + (l + 1)) + " but got: " + tm.getText());
+                    }
+                }
+                consumerSession.commit();
+            } finally {
+                consumingConnection.close();
+                producingConnection.close();
+            }
+        }
+    }
 }
